class MandatoryArgsNotFilled(Exception):
    """Exception raised for errors in the request args. This method expects to be fulfilled all the mandatory params."""

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)


class ContentTypeNotSupported(Exception):
    """Exception raised for errors in the body request type. The request's body should be formatted as a JSON."""

    def __init__(self, message):
        self.message = message
        super().__init__(self.message)



"""
This library contains methods to check and help with http requests.

"""

import json
from rudderclient.request.exceptions import (
    ContentTypeNotSupported,
    MandatoryArgsNotFilled,
)


def check_content_type(request, available_content_type):
    """
    Check if the content_type of the request is the content_type available in the request.

    Parameters:
    ----------
        request: The request.
        available_content_type: The available content type for the request.

    Raise:
    -----
    An exception of type `ContentTypeNotSupported`
    """
    if request.headers.get("Content-Type") != available_content_type:
        raise ContentTypeNotSupported(
            "The request's body should be formatted as a JSON."
        )


def get_params(request, param):
    """
    Returns the value of a specific param in the request.

    Parameters:
    ----------
        request: The request.
        param: The name of the param in the request.

    Raise:
    -----
    An exception of type `MandatoryArgsNotFilled` if the param asked is not in the request.
    """
    data = json.loads(request.data)

    if param in data:
        return data.get(param)
    else:
        raise MandatoryArgsNotFilled("Unexpected body parameters.")



"""
HTTP requests library.

This library provides an easy way to do HTTP request.

"""


import http.client
import requests

import google.auth.transport.requests
import google.oauth2.id_token


def iap_http_request(
    method: str,
    api_url: str,
    request_url: str,
    body: str,
    headers: dict[str, str],
):
    """
    Makes an http request to an API protected by Identity-Aware Proxy.

    Parameters:
    ----------
        method: Method of the request. "POST", "PATCH", "DELETE", "GET", "PUT".
        api_url: The DNS of the API. For example "api-dev.api.test.cloud"
        request_url: The request url. For example "/api/v1/users"
        body: Body of the requests.
        headers: Necessary headers for the requests. The "Authentication: Bearer {token}" token
        is mandatory. The token value is an OIDC token.
    """
    conn = http.client.HTTPSConnection(api_url)
    conn.request(method=method, url=request_url, body=body, headers=headers)
    res = conn.getresponse()
    data = res.read()
    print(data.decode("utf-8"))
    return data.decode("utf-8")


def make_authorized_post_request(endpoint, audience, body):
    """
    Makes a POST request to another HTTP Cloud Run
    by authenticating with the ID token obtained from the google-auth client library
    using the specified audience value.

    Parameters:
    ----------
        endpoint: The URL (hostname + path) receiving the request. For example: 'https://my-cloud-run-service.run.app/my/awesome/url'
        audience: Your service's hostname. For example: 'https://my-cloud-run-service.run.app/'
        body: The body of the request.
    """

    auth_req = google.auth.transport.requests.Request()
    id_token = google.oauth2.id_token.fetch_id_token(auth_req, audience)

    headers = {"Authorization": f"Bearer {id_token}"}

    response = requests.request("POST", endpoint, headers=headers, json=body)

    print(response.request)

    return response.status_code



import json
import requests


def get_all_organizations(token):
    """
    List organizations for the authenticated user.

    Parameters:
        token: Personal Access Token of the user with the necessary permissions.
    """
    url = "https://api.github.com/user/orgs"
    token = token

    headers = {"Authorization": f"Bearer {token}"}

    response = requests.request("GET", url, headers=headers, data={})

    return response.json()


def remove_user_from_an_organization(org, username, token):
    """
    Remove an organization member.
    * Remove a user form a organization will remove it from all teams and they will no longer
    have any access to the organization's repositories.

    Parameters:
        org - string
            the organization name to remove from.
        username - string
            the username for the Github user account.
        token - string
            Personal Access Token of the user with the necessary permissions.
    """
    url = f"https://api.github.com/orgs/{org}/members/{username}"
    token = token

    headers = {"Authorization": f"Bearer {token}"}

    response = requests.request("DELETE", url, headers=headers, data={})

    return response.status_code


def get_team_id(org, slug, token):
    """
    Get the team integer id by its slug* .
    * The slug for Github is the name of the team, but replacing special characters, changes all words to lowercase, and
    replaces spaces with a - separator. For example: team name -> "My TEam Näme" slug -> "my-team-name"

    Parameters:
        org - string
            the organization name to remove from.
        slug - string
            the slug for the team name.
        token - string
            Personal Access Token of the user with the necessary permissions.
    """
    url = f"https://api.github.com/orgs/{org}/teams/{slug}"
    token = token

    headers = {"Authorization": f"Bearer {token}"}

    response = requests.request("GET", url, headers=headers, data={})

    # Parse to json the content response to get team id
    content = response.content
    content_string = content.decode("utf-8")
    json_content = json.loads(content_string)

    return json_content["id"]


def invite_users(github_user, org, teams, role, token):
    """
    Invite people to an organization. Also, when the invitation was accepted for the users, they will be a member
    for the teams indicated if any.

    Parameters:
        github_user - string
            email address of the user to invite.
        org - string
            the organization name to remove from.
        teams - array of integers
            the specify IDs for th teams you want to invite the user.
        role - string
            the organization role for the new member.
                * admin - Organization owners with full administrative rights to the organization
                and complete access to all repositories and teams.
                * direct_member - Non-owner organization members with ability to see other members
                and join teams by invitation.
                * billing_manager - Non-owner organization members with ability to manage the billing
                settings of your organization.
        token - string
            Personal Access Token of the user with the necessary permissions.
    """
    url = f"https://api.github.com/orgs/{org}/invitations"
    token = token

    headers = {"Authorization": f"Bearer {token}"}

    data = {"email": github_user, "role": role, "team_ids": teams}

    # Parse data to bytestring
    res_bytes = json.dumps(data).encode("utf-8")

    response = requests.request("POST", url, headers=headers, data=res_bytes)

    return response.status_code



"""
This library provides an easy way to interact with Pub/Sub API.

"""


from google.cloud import pubsub_v1


def write_message(project_id, topic_id, message_str, status, joiner_id):
    """
    Publish a message in Pub Sub.

    Parameters:
    ----------
        project_id: The ID of the gcp project where you want to publish a message.
        topic_id: The topic name where you want to publish the message.
        message_str: The message string to publish.
        status and joinerId: Attributes for the message.
    """
    publisher = pubsub_v1.PublisherClient()
    topic_path = publisher.topic_path(project_id, topic_id)

    # Data must be a bytestring
    message = message_str.encode("utf-8")

    future = publisher.publish(
        topic_path, message, status=str(status), joinerId=str(joiner_id)
    )

    result = future.result()

    return result



"""
This library provides an easy way to interact with workspace API.

"""

from googleapiclient.discovery import build
from google.oauth2 import service_account


def create_service(service_name, version, credentials):
    """
    Create the necessary service to interact with a specific Google Workspace service in a specific version of its API.

    Parameters:
    ----------
        serviceName: Name of the service.
        version: Version of the service.
        credentials: oauth2client.Credentials or google.auth.credentials.Credentials, credentials to be used for
            authentication. You can get them with 'get_workspace_impersonate_credentials_sa' method.

    """
    service = build(service_name, version, credentials=credentials)
    return service


def user_workspace_exist(credentials, work_email, domain):
    """
    Return true if the user email exists in the Google Workspce domain specificated, and false if not.

    Parameters:
    ----------
        credentials: oauth2client.Credentials or google.auth.credentials.Credentials, credentials to be used for
            authentication. You can get them with 'get_workspace_impersonate_credentials_sa' method.
        work_email: Primary email address of the user to search for.
        domain: The domain name where you want to search for.
    """
    service = create_service("admin", "directory_v1", credentials)
    result = (
        service.users()
        .list(query=work_email, orderBy="email", domain=domain)
        .execute()
    )

    if "users" in result and len(result["users"]) > 0:
        exists = True
    else:
        exists = False

    return exists


def create_workspace_user(
    name,
    surname,
    second_surname,
    personal_email,
    primary_email,
    password,
    credentials,
):
    """
    Create a user in Google workspace.

    Parameters:
    ----------
        name: Name of the user to create.
        surname: First surname of the user to create.
        second_surname: Second surname of the user to create.
        personal_email: Personal email of the user.
        primary_email: Primary email of the user.
        password: First password.
        credentials: oauth2client.Credentials or google.auth.credentials.Credentials, credentials to be used for
            authentication. You can get them with 'get_workspace_impersonate_credentials_sa' method.
    """
    service = create_service("admin", "directory_v1", credentials)

    user_json = {
        "primaryEmail": primary_email,
        "name": {
            "givenName": name,
            "familyName": surname + " " + second_surname,
            "fullName": name + " " + surname + " " + second_surname,
        },
        "password": password,
        "changePasswordAtNextLogin": "true",
        "emails": [
            {"address": personal_email, "type": "home"},
            {"address": primary_email, "type": "work", "primary": "true"},
        ],
        "languages": [
            {"languageCode": "es", "preference": "preferred"},
            {"languageCode": "en-GB", "preference": "preferred"},
        ],
        "orgUnitPath": "/",
        "includeInGlobalAddressList": "true",
    }
    service.users().insert(body=user_json).execute()


def add_member_workspace_group(primary_email, role, group, credentials):
    """
    Add a member into a Google Workspace group with the specified role.

    Parameters:
        primary_email:
        role: The member's role in the group.
            Acceptable values are: MANAGER, OWNER, MEMBER. Visit this link for more information https://developers.google.com/admin-sdk/directory/reference/rest/v1/members#Member
        group: Identifier for the group.
        credentials: oauth2client.Credentials or google.auth.credentials.Credentials, credentials to be used for
            authentication. You can get them with 'get_workspace_impersonate_credentials_sa' method.
    """
    service = create_service("admin", "directory_v1", credentials)

    member_json = {
        "email": primary_email,
        "role": role,
    }
    service.members().insert(body=member_json, groupKey=group).execute()


def update_workspace_license(
    current_product_id,
    current_sku_id,
    current_user_id,
    license_instance,
    credentials,
):
    """
    Update the Google Workspace license of a user already created.

    Parameters:
    ----------
        current_product_id: The current product's unique identifier. You can check it in this link https://developers.google.com/admin-sdk/licensing/v1/how-tos/products
        current_sku_id: The current product SKU's unique identifier. You can check it in this link https://developers.google.com/admin-sdk/licensing/v1/how-tos/products
        current_user_id: The user's current primary email address.
        license_instance: License object instance for the updated.
        Example: {
                    'productId': "Google-Apps",
                    'skuId': "1010020030",
                    'userId': xxxxxx
                    }
        credentials: oauth2client.Credentials or google.auth.credentials.Credentials, credentials to be used for
            authentication. You can get them with 'get_workspace_impersonate_credentials_sa' method.
    """
    service = create_service("licensing", "v1", credentials)
    service.licenseAssignments().update(
        productId=current_product_id,
        skuId=current_sku_id,
        userId=current_user_id,
        body=license_instance,
    ).execute()


def get_workspace_impersonate_credentials_sa(
    credentials_info, scopes, impersonate_mail
):
    """
    Obtain the credentials with the permissions of the service account, in an impersonal way.

    Parameters:
    ----------
        credentials_info: json access key of the service account.
        scopes: Array of the granular permissions that determine what specific resources and operations the service
                account can access.
        impersonate_mail: Email to impersonate the service account.

    Returns:
    -------
    The impersonate credentials with the service account and scope permissions.
    """
    credentials = service_account.Credentials.from_service_account_info(
        credentials_info
    )

    scoped_credentials = credentials.with_scopes(scopes)

    impersonate_credentials = scoped_credentials.with_subject(impersonate_mail)

    return impersonate_credentials



import logging
import json
from datetime import datetime
from logging import LoggerAdapter


class CustomJsonFormatter(logging.Formatter):
    def format(self, record):
        log_object = {
            "timestamp": datetime.utcnow().isoformat(),
            "level": record.levelname,
            "message": record.getMessage(),
            "filename": record.filename,
            "lineno": record.lineno,
            "funcName": record.funcName,
            "userUUID": getattr(record, "userUUID", ""),
            "correlationID": getattr(record, "correlationID", ""),
        }
        return json.dumps(log_object)


class CustomLoggerAdapter(LoggerAdapter):
    def process(self, msg, kwargs):
        return (
            "[%s] [%s] %s"
            % (self.extra["userUUID"], self.extra["correlationID"], msg),
            kwargs,
        )


def get_custom_logger(name, userUUID="", correlationID=""):
    handler = logging.StreamHandler()
    handler.setFormatter(CustomJsonFormatter())
    logger = logging.getLogger(name)
    logger.addHandler(handler)
    logger.setLevel(logging.INFO)
    return CustomLoggerAdapter(
        logger, {"userUUID": userUUID, "correlationID": correlationID}
    )



"""
GCP Authentication library.

This library provides an easy way for authenticating to GCP services. Also offers a simple method
to read a value secret interacting with secretmanager API.

"""


from google.cloud import secretmanager
from google.oauth2 import id_token
from google.auth.transport.requests import Request
from google.oauth2 import service_account


def get_secret(secret_id):
    """
    Returns a secret value from secret manager service.

    Parameters:
    ----------
        secret_id: str
            Secret resource name like projects/xxxxxx/secrets/xxxxxxxx/versions/latest

    Returns:
    -------
    The value of the secret.
    """
    client = secretmanager.SecretManagerServiceClient()
    secret = client.access_secret_version(request={"name": secret_id})
    value = secret.payload.data.decode("UTF-8")
    return value


def get_oidc_token_iap_request(client_id, **kwargs):
    """
    Returns a token for doing an HTTP request to an application protected by Identity-Aware Proxy.

    Parameters:
    ----------
        client_id: The client ID used by Identity-Aware Proxy
        **kwargs: Any of the parameters defined for the request function:
                https://github.com/requests/requests/blob/master/requests/api.py
                If no timeout is provided, it is set to 90 by default.

    Returns:
    -------
    An OpenID Connect (OIDC) token from metadata server.
    """
    # Set the default timeout, if missing
    if "timeout" not in kwargs:
        kwargs["timeout"] = 90

    # Obtain an OpenID Connect (OIDC) token from metadata server or using service
    # account.
    open_id_connect_token = id_token.fetch_id_token(Request(), client_id)

    return open_id_connect_token


def get_workspace_impersonate_credentials_sa(
    credentials_info, scopes, impersonate_mail
):
    """
    Obtain the credentials with the permissions of the service account, in an impersonal way.

    Parameters:
    ----------
        credentials_info: json access key of the service account.
        scopes: Array of the granular permissions that determine what specific resources and operations the service
                account can access.
        impersonate_mail: Email to impersonate the service account.

    Returns:
    -------
    The impersonate credentials with the service account and scope permissions.
    """
    credentials = service_account.Credentials.from_service_account_info(
        credentials_info
    )

    scoped_credentials = credentials.with_scopes(scopes)

    impersonate_credentials = scoped_credentials.with_subject(impersonate_mail)

    return impersonate_credentials



import os
import smtplib
from email.mime.multipart import MIMEMultipart
from email.mime.text import MIMEText
from email.mime.base import MIMEBase
from email.mime.image import MIMEImage
from email import encoders


def read_template(file, replacements):
    """
    Read an HTML file and replace placeholders with actual values.

    :param file: Path to the HTML file.
    :param replacements: A dictionary where the keys are the placeholder names and the values are the replacement strings.
    :return: The content of the HTML file with placeholders replaced.
    """
    try:
        with open(file, "r") as f:
            content = f.read()
        for key, value in replacements.items():
            content = content.replace("{{ " + key + " }}", value)
        return content
    except FileNotFoundError:
        print(f"Error: File {file} not found.")
        return None


def attach_files(msg, attachment_dir):
    """
    Attach files to the email message.

    :param msg: The email message object.
    :param attachment_dir: Path to the directory containing the files to attach.
    """
    for file_name in os.listdir(attachment_dir):
        file_path = os.path.join(attachment_dir, file_name)
        try:
            with open(file_path, "rb") as f:
                if file_name.endswith("logo.png"):
                    img = MIMEImage(f.read())
                    img.add_header(
                        "Content-Disposition",
                        "attachment",
                        filename="logo.png",
                    )
                    img.add_header("Content-ID", "<logo.png>")
                    msg.attach(img)
                else:
                    part = MIMEBase("application", "octet-stream")
                    part.set_payload(f.read())
                    encoders.encode_base64(part)
                    part.add_header(
                        "Content-Disposition", "attachment", filename=file_name
                    )
                    msg.attach(part)
        except FileNotFoundError:
            print(f"Error: Attachment file {file_path} not found.")


def send_email(
    smtp_username,
    smtp_password,
    from_address,
    to_address,
    subject,
    cc_address=None,
    html_file=None,
    attachment_dir=None,
    replacements=None,
):
    """
    Send an email using AWS SMTP.

    :param smtp_username: The AWS SMTP username.
    :param smtp_password: The AWS SMTP password.
    :param from_address: The sender's email address.
    :param to_address: The recipient's email address.
    :param subject: The email subject.
    :param cc_address: (Optional) The CC email address.
    :param html_file: (Optional) Path to the HTML file to use as the email body.
    :param attachment_dir: (Optional) Path to the directory containing the files to attach.
    :param replacements: (Optional) A dictionary where the keys are the placeholder names in the HTML file and the values are the replacement strings.
    """
    try:
        with smtplib.SMTP("email-smtp.us-east-1.amazonaws.com", 587) as server:
            server.starttls()
            server.login(smtp_username, smtp_password)

            msg = MIMEMultipart()
            msg["From"] = from_address
            msg["To"] = to_address
            msg["Subject"] = subject
            if cc_address:
                msg["Cc"] = cc_address

            if html_file and replacements:
                html = read_template(html_file, replacements)
                if html:
                    msg.attach(MIMEText(html, "html"))

            if attachment_dir:
                attach_files(msg, attachment_dir)

            text = msg.as_string()
            server.sendmail(from_address, to_address, text)
    except smtplib.SMTPException as e:
        print(f"Error sending email: {e}")



"""This library provides an easy way to interact with boto3, the official Python library of AWS."""


import boto3


def get_boto3_client(resource, region, acces_key_id, secret_access_key):
    """
    Returns a low-level service client.

    Parameters:
    ----------
        resource: The name of a service, e.g 'identitystore'. You can get a list of available
                services via get_available_services().
        region: The name of the region associated with the client. A client is associated with a single region.
        access_key_id: The access key to use when creating the client.
        secret_access_key: The secret key to use when creating the client.
    """
    client = boto3.client(
        resource,
        region_name=region,
        aws_access_key_id=acces_key_id,
        aws_secret_access_key=secret_access_key,
    )
    return client


def delete_user(operation_parameters, region, acces_key_id, secret_access_key):
    """
    Deletes a user of a unique identityStore.

    Parameters:
    ----------
        operation_parameters: A dict with the IdentityStoreId and the UserId you want to delete.
                            Example: {'IdentityStoreId': identityStoreId, 'UserId': user_id}
        region: The name of the region associated with the identityStore.
        access_key_id: The access key to use when creating the client.
        secret_access_key: The secret key to use when creating the client.
    """
    # Create IAM client
    client = get_boto3_client(
        "identitystore", region, acces_key_id, secret_access_key
    )

    delete_response = client.delete_user(**operation_parameters)

    return delete_response


def get_identitystore_id(region, acces_key_id, secret_access_key):
    """
    Returns the ID of your organization Identity Store in a unique region.

    Parameters:
    ----------
        region: The name of the region associated with the identityStore.
        access_key_id: The access key to use when creating the client.
        secret_access_key: The secret key to use when creating the client.
    """
    client = get_boto3_client(
        "sso-admin", region, acces_key_id, secret_access_key
    )
    identitystore = client.list_instances()
    identitystore_id = identitystore["Instances"][0]["IdentityStoreId"]
    return identitystore_id


def describe_user(user_id, region, acces_key_id, secret_access_key):
    """
    Returns information of a user.

    Parameters:
    ----------
        user_id: The ID of the user.
        region: The name of the region associated with the identityStore.
        access_key_id: The access key to use when creating the client.
        secret_access_key: The secret key to use when creating the client.

    """
    client = get_boto3_client(
        "identitystore", region, acces_key_id, secret_access_key
    )
    identitystore = get_identitystore_id(
        region, acces_key_id, secret_access_key
    )
    response = client.describe_user(
        IdentityStoreId=identitystore, UserId=user_id
    )
    return response


def create_user(
    user_email, first_name, surnames, region, acces_key_id, secret_access_key
):
    """
    Creates a user in an identity Store.

    Parameters:
    ----------
        userEmail: the work email of the user.
        firstName: Name of the user.
        surnames: One or two surnames of the user.
        region: The name of the region associated with the identityStore.
        access_key_id: The access key to use when creating the client.
        secret_access_key: The secret key to use when creating the client.
    """
    client = get_boto3_client(
        "identitystore", region, acces_key_id, secret_access_key
    )
    identitystore = get_identitystore_id(
        region, acces_key_id, secret_access_key
    )
    display_name = f"{first_name} {surnames}"

    response = client.create_user(
        IdentityStoreId=identitystore,
        UserName=user_email,
        DisplayName=display_name,
        Name={"FamilyName": first_name, "GivenName": surnames},
        Emails=[{"Value": user_email, "Type": "Work", "Primary": True}],
    )
    return response


def pagination(
    action, operation_parameters, region, acces_key_id, secret_access_key
):
    """
    Paginates a list result.

    Parameters:
    ----------
        action: The action you want to paginate. For example, 'list_users'.
        operation_parameters: Necessary parameters for doing the action.
                            For example {'IdentityStoreId': identityStoreId, 'UserId': user_id}
        region: The name of the region associated with the identityStore.
        access_key_id: The access key to use when creating the client.
        secret_access_key: The secret key to use when creating the client.
    """
    # Create IAM client
    client = get_boto3_client(
        "identitystore", region, acces_key_id, secret_access_key
    )

    paginator = client.get_paginator(action)
    page_iterator = paginator.paginate(**operation_parameters)

    return page_iterator
